@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2002 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,16 +16,34 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * begin      : Wed Mar 21 2001
- * description: This file contains all the general things.
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
+ */
+
+
+/*
+ *                         common.h  -  description
+ *                         ------------------------
+ *   begin                : Sat Dec 23 2000
+ *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   description          : This file contains things common to the whole soft.
+ *
  */
 
 
 #ifndef GM_COMMON_H_
 #define GM_COMMON_H_
 
-#include "../config.h"
+#include <openh323buildopts.h>
+#include <ptbuildopts.h>
 
+#include <ptlib.h>
+#include <h323.h>
 
 #ifndef DISABLE_GNOME
 #include <gnome.h>
@@ -33,13 +51,28 @@
 #include <gtk/gtk.h>
 #endif
 
-#include <ptlib.h>
-#include <iostream>
+#ifndef DISABLE_GCONF
+#include <gconf/gconf-client.h>
+#else
+#include "../lib/win32/gconf-simu.h"
+#endif
+
+#ifdef WIN32
+#include <string.h>
+#define strcasecmp strcmpi
+#define vsnprintf _vsnprintf
+#endif
+
+#include "menu.h"
+
 
 #define GENERAL_KEY         "/apps/gnomemeeting/general/"
+#define PORTS_KEY           "/apps/gnomemeeting/ports/"
 #define VIEW_KEY            "/apps/gnomemeeting/view/"
+#define VIDEO_DISPLAY_KEY   "/apps/gnomemeeting/video_display/"
 #define DEVICES_KEY         "/apps/gnomemeeting/devices/"
 #define PERSONAL_DATA_KEY   "/apps/gnomemeeting/personal_data/"
+#define CALL_FORWARDING_KEY "/apps/gnomemeeting/call_forwarding/"
 #define LDAP_KEY            "/apps/gnomemeeting/ldap/"
 #define GATEKEEPER_KEY      "/apps/gnomemeeting/gatekeeper/"
 #define SERVICES_KEY        "/apps/gnomemeeting/services/"
@@ -50,7 +83,8 @@
 #define DEVICES_KEY         "/apps/gnomemeeting/devices/"
 #define CONTACTS_KEY        "/apps/gnomemeeting/contacts/"
 #define CONTACTS_GROUPS_KEY "/apps/gnomemeeting/contacts/groups/"
-
+#define HISTORY_KEY         "/apps/gnomemeeting/history/"
+#define CALL_FORWARDING_KEY "/apps/gnomemeeting/call_forwarding/"
 
 #define GM_CIF_WIDTH   352
 #define GM_CIF_HEIGHT  288
@@ -64,11 +98,7 @@
 
 #define GM_MAIN_NOTEBOOK_HIDDEN 4
 
-#ifdef SPEEX_CODEC
 #define GM_AUDIO_CODECS_NUMBER 9
-#else
-#define GM_AUDIO_CODECS_NUMBER 7
-#endif
 
 #define GNOMEMEETING_PAD_SMALL 1
 
@@ -90,10 +120,11 @@
 typedef struct _GmWindow GmWindow;
 typedef struct _GmPrefWindow GmPrefWindow;
 typedef struct _GmLdapWindow GmLdapWindow;
+typedef struct _GmLdapWindowPage GmLdapWindowPage;
 typedef struct _GmTextChat GmTextChat;
 typedef struct _GmDruidWindow GmDruidWindow;
+typedef struct _GmCallsHistoryWindow GmCallsHistoryWindow;
 typedef struct _GmRtpData GmRtpData;
-typedef struct _GmCommandLineOptions GmCommandLineOptions;
 
 enum {CONTACTS_SERVERS, CONTACTS_GROUPS};
 
@@ -152,12 +183,10 @@ struct _GmWindow
   GtkWidget *ldap_window;
   GtkWidget *chat_window;
   GtkWidget *calls_history_window;
-  GtkWidget *calls_history_text_view;
   GtkWidget *preview_button;
   GtkWidget *connect_button;
   GtkWidget *video_chan_button;
   GtkWidget *audio_chan_button;
-  GtkWidget *speaker_phone_button;
   GtkWidget *incoming_call_popup;
   GtkWidget *stats_label;
   GtkWidget *stats_drawing_area;
@@ -169,7 +198,6 @@ struct _GmWindow
   GdkColor colors [6];
 
   int          progress_timeout;
-  int          cleaner_thread_count;
 
   PStringArray video_devices;
   PStringArray audio_recorder_devices;
@@ -180,12 +208,29 @@ struct _GmWindow
 
 struct _GmLdapWindow
 {
+  MenuEntry *addressbook_menu;
   GtkWidget *notebook;
   GtkWidget *tree_view;
   GtkWidget *option_menu;
 };
 
 
+struct _GmLdapWindowPage
+{
+  GtkWidget *section_name;
+  GtkWidget *tree_view;
+  GtkWidget *statusbar;
+  GtkWidget *option_menu;
+  GtkWidget *search_entry;
+
+  PThread *ils_browser;
+  PMutex search_quit_mutex;
+  
+  gchar *contact_section_name;
+  gint page_type;
+};
+
+
 struct _GmDruidWindow
 {
 #ifndef DISABLE_GNOME
@@ -210,6 +255,14 @@ struct _GmDruidWindow
 };
 
 
+struct _GmCallsHistoryWindow
+{
+  GtkListStore *given_calls_list_store;
+  GtkListStore *received_calls_list_store;
+  GtkListStore *missed_calls_list_store;
+};
+
+
 struct _GmPrefWindow
 {
   GtkTooltips  *tips;
@@ -218,6 +271,7 @@ struct _GmPrefWindow
   GtkWidget    *start_hidden;
   GtkWidget    *incoming_call_popup;
   GtkWidget    *incoming_call_sound;
+  GtkWidget    *auto_clear_text_chat;
   GtkWidget    *opt1, *opt2;
   GtkWidget    *tr_vq;
   GtkWidget    *maximum_video_bandwidth;
@@ -272,61 +326,4 @@ struct _GmPrefWindow
   GtkWidget    *ip_translation;
   GtkWidget    *public_ip;
 };
-
-
-struct _GmCommandLineOptions
-{
-  int    debug_level;
-  gchar *url;
-  int    daemon;
-};
-
-
-/* DESCRIPTION  :  / 
- * BEHAVIOR     :  Returns the structure of widgets of the main window.
- * PRE          :  The GtkWidget must be a pointer to the Main gnomeMeeting 
- *                 window.
- */
-GmWindow *gnomemeeting_get_main_window (GtkWidget *);
-
-
-/* DESCRIPTION  :  / 
- * BEHAVIOR     :  Returns the structure of widgets of the prefs window.
- * PRE          :  The GtkWidget must be a pointer to the Main gnomeMeeting 
- *                 window.
- */
-GmPrefWindow *gnomemeeting_get_pref_window (GtkWidget *);
-
-
-/* DESCRIPTION  :  / 
- * BEHAVIOR     :  Returns the structure of widgets of the ldap window.
- * PRE          :  The GtkWidget must be a pointer to the Main gnomeMeeting 
- *                 window.
- */
-GmLdapWindow *gnomemeeting_get_ldap_window (GtkWidget *);
-
-
-/* DESCRIPTION  :  / 
- * BEHAVIOR     :  Returns the structure of widgets of the chat window.
- * PRE          :  The GtkWidget must be a pointer to the Main gnomeMeeting 
- *                 window.
- */
-GmTextChat *gnomemeeting_get_chat_window (GtkWidget *);
-
-
-/* DESCRIPTION  :  / 
- * BEHAVIOR     :  Returns the structure of widgets of the druid window.
- * PRE          :  The GtkWidget must be a pointer to the Main gnomeMeeting 
- *                 window.
- */
-GmDruidWindow *gnomemeeting_get_druid_window (GtkWidget *);
-
-
-/* DESCRIPTION  :  / 
- * BEHAVIOR     :  Returns the rtp data. Only valid during calls.
- * PRE          :  The GtkWidget must be a pointer to the Main gnomeMeeting 
- *                 window.
- */
-GmRtpData *gnomemeeting_get_rtp_data (GtkWidget *);
-
 #endif /* GM_COMMON_H */
