@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2001 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,38 +15,35 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         druid.cpp  -  description
  *                         --------------------------
  *   begin                : Mon May 1 2002
- *   copyright            : (C) 2000-2002 by Damien Sandras
+ *   copyright            : (C) 2000-2003 by Damien Sandras
  *   description          : This file contains all the functions needed to
  *                          build the druid.
- *   email                : dsandras@seconix.com
  */
 
 
 #include "../config.h"
 
-#include <iostream>
-#include <gtk/gtk.h>
-#include <ptlib.h>
-
-#ifndef DISABLE_GNOME
-#include <gnome.h>
-#endif
-
-#include <gconf/gconf-client.h>
-
+#include "druid.h"
 #include "gnomemeeting.h"
-#include "dialog.h"
-#include "misc.h"
 #include "ils.h"
-#include "videograbber.h"
+#include "misc.h"
+#include "dialog.h"
 #include "stock-icons.h"
-#include "callbacks.h"
 
 
 #ifndef DISABLE_GNOME
@@ -59,7 +56,6 @@ static void gnomemeeting_druid_cancel (GtkWidget *, gpointer);
 static void gnomemeeting_druid_quit (GtkWidget *, gpointer);
 static void gnomemeeting_druid_destroy (GtkWidget *, GdkEventAny *, gpointer);
 static void gnomemeeting_druid_user_page_check (GnomeDruid *);
-static void gnomemeeting_druid_toggle_changed (GtkToggleButton *, gpointer);
 static void gnomemeeting_druid_entry_changed (GtkWidget *, gpointer);
 static void gnomemeeting_druid_radio_changed (GtkToggleButton *, gpointer);
 static void gnomemeeting_druid_page_prepare (GnomeDruidPage *, GnomeDruid *,
@@ -140,8 +136,8 @@ gnomemeeting_druid_cancel (GtkWidget *w, gpointer data)
   GmWindow *gw = NULL;
   GmDruidWindow *dw = NULL;
 
-  gw = gnomemeeting_get_main_window (gm);
-  dw = gnomemeeting_get_druid_window (gm);
+  gw = MyApp->GetMainWindow ();
+  dw = MyApp->GetDruidWindow ();
 
   gnome_druid_set_page (dw->druid, GNOME_DRUID_PAGE (dw->page_edge));
   gtk_widget_hide (gw->druid_window);
@@ -168,8 +164,8 @@ gnomemeeting_druid_quit (GtkWidget *w, gpointer data)
   
   client = gconf_client_get_default ();
 
-  gw = gnomemeeting_get_main_window (gm);
-  dw = gnomemeeting_get_druid_window (gm);
+  gw = MyApp->GetMainWindow ();
+  dw = MyApp->GetDruidWindow ();
 
 
   /* Always register to make the callto available,the user can choose
@@ -185,11 +181,6 @@ gnomemeeting_druid_quit (GtkWidget *w, gpointer data)
 				FALSE);
 
 
-  /**/
-  gconf_client_set_int (client, GENERAL_KEY "version", 
-			MAJOR_VERSION * 100 + MINOR_VERSION, 0);
-
-
   /* Connection type */
   group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dw->kind_of_net));
   while (group) {
@@ -217,8 +208,6 @@ gnomemeeting_druid_quit (GtkWidget *w, gpointer data)
       gconf_client_set_bool (client, SERVICES_KEY "enable_microtelco",
 			     true, 0);
 
-      MyApp->Endpoint ()->SetUserNameAndAlias ();
-      MyApp->Endpoint ()->RemoveGatekeeper (0);
       MyApp->Endpoint ()->GatekeeperRegister ();
     }
 
@@ -259,8 +248,7 @@ gnomemeeting_druid_quit (GtkWidget *w, gpointer data)
 	&& registering_method == 1) {
 
       gconf_client_set_int (client, GATEKEEPER_KEY "registering_method", 0, 0);
-      MyApp->Endpoint ()->SetUserNameAndAlias ();
-      MyApp->Endpoint ()->RemoveGatekeeper (0);
+      MyApp->Endpoint ()->GatekeeperRegister ();
     }
 
     g_free (gk_name);
@@ -301,7 +289,7 @@ gnomemeeting_druid_user_page_check (GnomeDruid *druid)
   
   /* Fetch data */
   client = gconf_client_get_default ();
-  dw = gnomemeeting_get_druid_window (gm);
+  dw = MyApp->GetDruidWindow ();
   
 
   /* We check that all fields are present, or the toggle desactivated */
@@ -330,30 +318,6 @@ gnomemeeting_druid_user_page_check (GnomeDruid *druid)
 }
 
 
-/* DESCRIPTION  :  Called when the registering toggle changes in the Personal
- *                 Information page.
- * BEHAVIOR     :  Checks if the "Next" button of the "Personal Information"
- *                 druid page can be sensitive or not. It will if all fields
- *                 are ok, or if registering is disabled. (Calls the above
- *                 function).
- * PRE          :  /
- */
-static void
-gnomemeeting_druid_toggle_changed (GtkToggleButton *button, gpointer data)
-{
-  GmDruidWindow *dw = NULL;
-  GmWindow *gw = NULL;
-
-  dw = gnomemeeting_get_druid_window (gm);
-  gw = gnomemeeting_get_main_window (gm);
-
-  gnomemeeting_druid_user_page_check (dw->druid);
-
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
-    gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gw->druid_window), GTK_WIDGET (button), _("You chose to NOT use the GnomeMeeting ILS directory. Other users will not be able to contact you if you don't register to a directory service."));
-}
-
-
 /* DESCRIPTION  :  Called when the user changes an info in the Personal
  *                 Information page.
  * BEHAVIOR     :  Checks if the "Next" button of the "Personal Information"
@@ -367,7 +331,7 @@ gnomemeeting_druid_entry_changed (GtkWidget *w, gpointer data)
 {
   GmDruidWindow *dw = NULL;
 
-  dw = gnomemeeting_get_druid_window (gm);
+  dw = MyApp->GetDruidWindow ();
 
   gnomemeeting_druid_user_page_check (dw->druid);
 }
@@ -386,8 +350,8 @@ gnomemeeting_druid_radio_changed (GtkToggleButton *b, gpointer data)
   GmWindow *gw = NULL;
   GmDruidWindow *dw = NULL;
   
-  gw = gnomemeeting_get_main_window (gm);
-  dw = gnomemeeting_get_druid_window (gm);
+  gw = MyApp->GetMainWindow ();
+  dw = MyApp->GetDruidWindow ();
   client = gconf_client_get_default ();
 
   /* Dialup */
@@ -469,7 +433,7 @@ static void
 gnomemeeting_druid_page_prepare (GnomeDruidPage *page, GnomeDruid *druid,
 				 gpointer data)
 {
-  GmDruidWindow *dw = gnomemeeting_get_druid_window (gm);
+  GmDruidWindow *dw = MyApp->GetDruidWindow ();
   GConfClient *client = gconf_client_get_default ();
   
   int kind_of_net = 1, cpt = 1;
@@ -534,7 +498,7 @@ gnomemeeting_druid_final_page_prepare (GnomeDruid *druid)
 
   /* Fetch the data */
   GConfClient *client = gconf_client_get_default ();
-  dw = gnomemeeting_get_druid_window (gm);
+  dw = MyApp->GetDruidWindow ();
 
   group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (dw->kind_of_net));
   while (group) {
@@ -597,7 +561,7 @@ gnomemeeting_druid_final_page_prepare (GnomeDruid *druid)
   microtelco =
     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dw->enable_microtelco));
   
-  callto = g_strdup_printf ("callto://ils.seconix.com/%s", mail);
+  callto = g_strdup_printf ("callto:ils.seconix.com/%s", mail);
   
   text = g_strdup_printf (_("You have now finished the GnomeMeeting configuration. All the settings can be changed in the GnomeMeeting preferences. Enjoy!\n\n\nConfiguration Summary:\n\nUsername:  %s %s\nConnection type:  %s\nAudio player:  %s\nAudio recorder:  %s\nVideo player: %s\nMy Callto URL: %s\nPC-To-Phone calls: %s"), firstname, lastname, kind_of_net_name, audio_player, audio_recorder, video_recorder, callto, microtelco ? _("Enabled") : _("Disabled"));
   gnome_druid_page_edge_set_text (page_final, text);
@@ -629,7 +593,7 @@ gnomemeeting_init_druid_user_page (GnomeDruid *druid, int p, int t)
   GnomeDruidPageStandard *page_standard = NULL;
 
 
-  dw = gnomemeeting_get_druid_window (gm);
+  dw = MyApp->GetDruidWindow ();
   page_standard = 
     GNOME_DRUID_PAGE_STANDARD (gnome_druid_page_standard_new ());
 
@@ -676,7 +640,7 @@ gnomemeeting_init_druid_user_page (GnomeDruid *druid, int p, int t)
   table = gnomemeeting_vbox_add_table (vbox, _("Directory of Online GnomeMeeting Users"), 1, 2);
   dw->ils_register =
     gnomemeeting_table_add_toggle (table, _("Publish my information on the directory of online GnomeMeeting users"), 
-				  LDAP_KEY "visible", NULL, 2, 0);
+				  LDAP_KEY "visible", NULL, 2);
     
 
   
@@ -707,7 +671,7 @@ gnomemeeting_init_druid_connection_type_page (GnomeDruid *druid, int p, int t)
 
   GConfClient *client = NULL;
 
-  GmDruidWindow *dw = gnomemeeting_get_druid_window (gm);
+  GmDruidWindow *dw = MyApp->GetDruidWindow ();
   GnomeDruidPageStandard *page_standard = NULL;
 
 
@@ -797,8 +761,8 @@ gnomemeeting_init_druid_audio_devices_page (GnomeDruid *druid, int p, int t)
 
   GConfClient *client = gconf_client_get_default ();
 
-  GmDruidWindow *dw = gnomemeeting_get_druid_window (gm);
-  GmWindow *gw = gnomemeeting_get_main_window (gm);
+  GmDruidWindow *dw = MyApp->GetDruidWindow ();
+  GmWindow *gw = MyApp->GetMainWindow ();
   
   gchar *title = NULL;
   gchar *player = NULL;
@@ -878,8 +842,8 @@ gnomemeeting_init_druid_video_devices_page (GnomeDruid *druid, int p, int t)
 
   GnomeDruidPageStandard *page_standard = NULL;
 
-  GmWindow *gw = gnomemeeting_get_main_window (gm);
-  GmDruidWindow *dw = gnomemeeting_get_druid_window (gm);
+  GmWindow *gw = MyApp->GetMainWindow ();
+  GmDruidWindow *dw = MyApp->GetDruidWindow ();
 
   page_standard = 
     GNOME_DRUID_PAGE_STANDARD (gnome_druid_page_standard_new ());
@@ -938,7 +902,7 @@ gnomemeeting_init_druid_ixj_device_page (GnomeDruid *druid, int p, int t)
   GtkWidget *vbox = NULL;
   GtkWidget *label = NULL;
 
-  GmDruidWindow *dw = gnomemeeting_get_druid_window (gm);
+  GmDruidWindow *dw = MyApp->GetDruidWindow ();
     
   GConfClient *client = gconf_client_get_default ();
 
@@ -1019,27 +983,23 @@ gnomemeeting_init_druid_ixj_device_page (GnomeDruid *druid, int p, int t)
 
 
 /* Functions */
-void 
-gnomemeeting_init_druid ()
+GtkWidget *
+gnomemeeting_druid_window_new (GmDruidWindow *dw)
 {
+  GtkWidget *window = NULL;
+  
 #ifndef DISABLE_GNOME
-  GmWindow *gw = NULL;
-  GmDruidWindow *dw = NULL;
-
   gchar *title = NULL;
 
   GnomeDruidPageEdge *page_final = NULL;
-
-  gw = gnomemeeting_get_main_window (gm);
-  dw = gnomemeeting_get_druid_window (gm);
   
-  gw->druid_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (gw->druid_window), 
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (window), 
 			_("First Time Configuration Druid"));
-  gtk_window_set_position (GTK_WINDOW (gw->druid_window), GTK_WIN_POS_CENTER);
+  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
   dw->druid = GNOME_DRUID (gnome_druid_new ());
 
-  gtk_container_add (GTK_CONTAINER (gw->druid_window), GTK_WIDGET (dw->druid));
+  gtk_container_add (GTK_CONTAINER (window), GTK_WIDGET (dw->druid));
 
 
   title = g_strdup_printf (_("Configuration Druid - page %d/%d"), 1, 7);
@@ -1049,7 +1009,7 @@ gnomemeeting_init_druid ()
      "The following steps will set up GnomeMeeting by asking "
      "a few simple questions.\n\nOnce you have completed "
      "these steps, you can always change them later by "
-     "selecting Preferences in the Edit menu. ");
+     "selecting Preferences in the Edit menu.");
 
 
   /* Create the first page */
@@ -1102,8 +1062,10 @@ gnomemeeting_init_druid ()
   g_signal_connect (G_OBJECT (dw->druid), "cancel",
 		    G_CALLBACK (gnomemeeting_druid_cancel), NULL);
 
-  g_signal_connect (G_OBJECT (gw->druid_window), "delete_event",
+  g_signal_connect (G_OBJECT (window), "delete_event",
 		    G_CALLBACK (gnomemeeting_druid_destroy), NULL);
 #endif
+
+  return window;
 }
 
