@@ -46,11 +46,6 @@
 #include "dialog.h"
 
 
-#ifdef __FreeBSD__
-#include <sys/types.h>
-#include <signal.h>
-#endif
-
 #ifdef HAS_IXJ
 #include <ixjlid.h>
 #endif
@@ -59,6 +54,13 @@
 #include <linux/soundcard.h>
 #endif
 
+#ifdef __FreeBSD__
+#if (__FreeBSD__ >= 5)
+#include <sys/soundcard.h>
+#else
+#include <machine/soundcard.h>
+#endif
+#endif
 
 
 static void dialog_response_cb (GtkWidget *, gint, gpointer);
